import sys
from loguru import logger


logger.remove(0)
logger.add(
    sys.stderr,
    format="<blue>[{time:HH:mm:ss}]</blue> <yellow>({level})</yellow> <red>|→</red> <green>{message}</green>",
    colorize=True,
    level="INFO",
    filter=lambda record: record["level"].name == "INFO",
)

logger.add(
    sys.stderr,
    format="<blue>[{time:HH:mm:ss}]</blue> <yellow>({level})</yellow> |> <yellow>{message}</yellow>",
    colorize=True,
    level="WARNING",
    filter=lambda record: record["level"].name == "WARNING",
)

logger.add(
    sys.stderr,
    format="<blue>[{time:HH:mm:ss}]</blue> <red>({level})|→ {message}</red>",
    colorize=True,
    level="ERROR",
    filter=lambda record: record["level"].name == "ERROR",
)
log = logger



from sys import path
from pathlib import Path
from .logger import log


async def load_cogs(bot, folder: str = "src", utils_folder: str = "utils") -> None:
    try:
        path.append(str(Path(folder)))

        for entry in Path(folder).iterdir():
            for filename in entry.iterdir():
                if filename.suffix == ".py" and entry.name != utils_folder:
                    bot.load_extension(f"{entry.name}.{filename.stem}")

        log.info(f"[Discord] Cogs success loaded")

    except Exception as e:
        log.error(f"[Discord] An error occurred while loading modules\n╰─> Error: {e}")



from random import choices
from dataclasses import dataclass, field
from string import ascii_uppercase, ascii_letters, digits
from typing import List, Dict, Any, Generator


@dataclass
class CodeArray:
    m_code: str
    m_name: str
    m_type: str
    m_itemsArray: List[Dict[str, Any]] = field(default_factory=list)
    m_teleport_position: str = ""
    m_vehicles: Dict[str, Any] = field(default_factory=lambda: {"m_item": "", "m_attachments": [], "m_cargo": []})
    m_give_zone_positions: List[Dict[str, Any]] = field(default_factory=list)
    m_give_zone_positions_forbidden: List[Dict[str, Any]] = field(default_factory=list)


class IssueHotline:
    MAX_STACK = 100

    @staticmethod
    def __generate_code() -> str:
        return "-".join([
            "".join(choices(ascii_uppercase, k=2)),
            "".join(choices(ascii_letters + digits, k=4)),
            "".join(choices(ascii_letters + digits, k=4))
        ])

    def __generate_count(self, items_list: List[Dict[str, Any]]) -> Generator[Dict[str, Any], None, None]:
        for item in items_list:
            item_count = item.get("m_count", 0)
            item_name = item.get("m_item", "")
            full_stacks, remainder = divmod(item_count, self.MAX_STACK)

            for _ in range(full_stacks):
                yield {"m_item": item_name, "m_count": self.MAX_STACK}

            if remainder > 0:
                yield {"m_item": item_name, "m_count": remainder}

    async def create_items(self, issue_name: str, items_list: List[Dict[str, Any]]) -> Dict[str, Any]:
        processed_items = [item for item in self.__generate_count(items_list)]
        return {"m_CodeArray": [CodeArray(
            m_code=self.__generate_code(),
            m_name=issue_name,
            m_type="item",
            m_itemsArray=processed_items
        ).__dict__]}

    async def create_vehicle(self, issue_name: str, items_dict: Dict[str, Any]) -> Dict[str, Any]:
        return {"m_CodeArray": [CodeArray(
            m_code=self.__generate_code(),
            m_name=issue_name,
            m_type="vehicle",
            m_vehicles=items_dict
        ).__dict__]}

    async def create_teleport(self, issue_name: str, teleport_position: str) -> Dict[str, Any]:
        return {"m_CodeArray": [CodeArray(
            m_code=self.__generate_code(),
            m_name=issue_name,
            m_type="teleport",
            m_teleport_position=teleport_position
        ).__dict__]}



from hashlib import sha256, md5
from base64 import b64encode


def steam_to_be_guid(steam_id: int) -> str:
    parts = [0x42, 0x45] + [0] * 8
    for i in range(2, 10):
        steam_id, remainder = divmod(steam_id, 256)
        parts[i] = remainder

    byte_array = bytes(parts)
    hash_object = md5(byte_array)
    return hash_object.hexdigest()


def steam_to_dayz_guid(steam_id: int) -> str:
    hashed = sha256()
    hashed.update(str(steam_id).encode("utf-8"))
    return b64encode(hashed.digest()).decode("utf-8")



from typing import Optional, Tuple, List, NoReturn
from disnake import TextChannel, User, Embed, Forbidden, HTTPException


class EmbedLog:

    @classmethod
    def build_embed(
        cls,
        title: Optional[str] = None,
        color: Optional[int] = None,
        fields: Optional[List[Tuple[str, str, bool]]] = None,
        desc: Optional[str] = None,
        url: Optional[str] = None,
        thumbnail: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
    ) -> Embed:
        """Создает Embed с заданными параметрами."""
        embed = Embed(title=title, description=desc, color=color, url=url)

        if fields:
            for name, value, inline in fields:
                embed.add_field(name=name, value=value, inline=inline)

        if thumbnail:
            embed.set_thumbnail(url=thumbnail)
        if image:
            embed.set_image(url=image)
        if footer:
            embed.set_footer(text=footer)

        return embed

    async def send_channel(
        self,
        channel: TextChannel,
        title: Optional[str] = None,
        color: Optional[int] = None,
        fields: Optional[List[Tuple[str, str, bool]]] = None,
        desc: Optional[str] = None,
        url: Optional[str] = None,
        thumbnail: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
        custom_embed: Optional[Embed] = None,
    ) -> None:
        embed = custom_embed or self.build_embed(
            title, color, fields, desc, url, thumbnail, image, footer
        )
        try:
            await channel.send(embed=embed)
        except Forbidden:
            print(f"[Ошибка] Нет прав на отправку сообщений в канал {channel.name}")
        except HTTPException as e:
            print(f"[Ошибка] Не удалось отправить сообщение: {e}")

    async def send_user(
        self,
        user: User,
        title: Optional[str] = None,
        color: Optional[int] = None,
        fields: Optional[List[Tuple[str, str, bool]]] = None,
        desc: Optional[str] = None,
        url: Optional[str] = None,
        thumbnail: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
        custom_embed: Optional[Embed] = None,
    ) -> None:
        embed = custom_embed or self.build_embed(
            title, color, fields, desc, url, thumbnail, image, footer
        )
        try:
            await user.send(embed=embed)
        except Exception:
            pass



from .tortoise import Database
from .logger import log
from .cogs import load_cogs

from .files import *

from .utils.misc import *
from .utils.hotline import *
from .utils.convert import *
from .utils.embed_log import *


__all__ = [
    "Database",
    "EmbedLog",

    "log",
    "load_cogs",

    "steam_to_be_guid",
    "steam_to_dayz_guid",

    "trans",
    "green",
    "red",
    "blue",
    "yellow",
    "purple",

    "restart",
    "get_user_id",

    "read_json",
    "write_json",
    "read_jsonc",
    "IssueHotline"
]



import re
import os
import sys
from disnake import Embed


red = 16711680
green = 65280
blue = 255
yellow = 16776960
purple = 9109759
trans = 2829617


def restart():
    python = sys.executable
    os.execl(python, python, '-B', *sys.argv)


def get_user_id(field_index: int, embed: Embed) -> int:
    user_id_str = re.search(r"<@!?(\d+)>", embed.fields[field_index].value).group(1)
    return int(user_id_str)



import re
import aiofiles

from typing import Optional, Any
from json import loads, dumps


async def read_json(path: str, *keys: str) -> Optional[Any]:
    try:
        async with aiofiles.open(path, "r", encoding="utf-8") as file:
            file_content = await file.read()
            file_data = loads(file_content)

            for key in keys:
                file_data = file_data.get(key)
                if file_data is None:
                    return None
            return file_data

    except Exception:
        raise


async def write_json(path: str, data: Any) -> bool:
    try:
        async with aiofiles.open(path, "w", encoding="utf-8") as file:
            await file.write(dumps(data, ensure_ascii=False, indent=2))
            return True
    except Exception:
        raise


async def read_jsonc(path: str, *keys: str) -> Optional[Any]:
    try:
        regex = re.compile(r"(\".*?\"|\'.*?\')|(/\*.*?\*/|//[^\r\n]*)", re.MULTILINE | re.DOTALL)

        def __re_sub(match):
            return match.group(1) or ""

        async with aiofiles.open(path, "r", encoding="utf-8") as file:
            file_content = regex.sub(__re_sub, await file.read())
            file_data = loads(file_content)

            for key in keys:
                file_data = file_data.get(key)
                if file_data is None:
                    return None
            return file_data
    except Exception:
        raise
